within ;
package Excercise12 "Modelling an electric light arc"
  model LightArc "A simple model of a light arc"
    extends Modelica.Electrical.Analog.Interfaces.OnePort;
    import Modelica.SIunits.*;

    // User parameters
    parameter Distance              gapDistance =        1.0e-2
      "Distance between the two contacts";
    parameter ElectricFieldStrength dielectricStrength = 2.5e+6
      "Minimum field strength for creating a light arc";
    parameter ElectricFieldStrength disconnectVoltage =  0.5e+6
      "Minimum field strength to sustain a light arc";
    parameter Current               minimalCurrent =     1.0e+0
      "Minimum current to sustain a light arc";

    // Shortcuts for later use
    final parameter Voltage    vOn =  dielectricStrength * gapDistance
      "Ignition voltage of the light arc";
    final parameter Voltage    vOff = disconnectVoltage  * gapDistance
      "Extinction voltage of the light arc";
    final parameter Resistance R =    vOff               / minimalCurrent
      "Extinction current of the light arc";

    // The state machine
    type ArcState = enumeration(
        off,
        positive,
        negative);
    ArcState arcState(start = ArcState.off);
  initial equation
    pre(arcState) = ArcState.off;
  equation
    arcState = if pre(arcState) == ArcState.off      and v >   +vOn then ArcState.positive elseif
                  pre(arcState) == ArcState.off      and v <   -vOn then ArcState.negative elseif
                  pre(arcState) == ArcState.positive and v <   vOff then ArcState.off      elseif
                  pre(arcState) == ArcState.negative and v >  -vOff then ArcState.off      else
                  pre(arcState);

    // The actual physical equation
    if     arcState == ArcState.off then
      i = 0;
    else
      v = R*i;
    end if;

    annotation (                                 Icon(coordinateSystem(
            preserveAspectRatio=false, extent={{-100,-100},{100,100}}), graphics={
          Polygon(
            points={{-100,6},{-60,6},{-60,20},{-56,20},{-56,-20},{-60,-20},{-60,-6},
                {-100,-6},{-100,6}},
            smooth=Smooth.None,
            pattern=LinePattern.None,
            lineColor={0,0,0},
            fillColor={135,135,135},
            fillPattern=FillPattern.Solid),
          Polygon(
            points={{-24,6},{16,6},{16,20},{20,20},{20,-20},{16,-20},{16,-6},{-24,
                -6},{-24,6}},
            smooth=Smooth.None,
            pattern=LinePattern.None,
            lineColor={0,0,0},
            fillColor={135,135,135},
            fillPattern=FillPattern.Solid,
            origin={76,0},
            rotation=180),
          Polygon(
            points={{-52,0},{-10,20},{10,-10},{52,0},{10,-20},{-10,10},{-52,0}},
            smooth=Smooth.None,
            fillColor={255,255,0},
            fillPattern=FillPattern.Solid,
            lineColor={255,0,0}),
          Text(
            extent={{-136,-60},{136,-92}},
            lineColor={0,0,0},
            textString="distance=%gapDistance"),
          Text(
            extent={{-150,85},{150,45}},
            textString="%name",
            lineColor={0,0,255})}));
  end LightArc;

  model testLightArc "Test our light arc model"

    Modelica.Electrical.Analog.Sources.ConstantCurrent constantCurrent(I=0.5)
      annotation (Placement(transformation(
          extent={{-10,-10},{10,10}},
          rotation=90,
          origin={-50,0})));
    Modelica.Electrical.Analog.Basic.Ground ground
      annotation (Placement(transformation(extent={{-60,-60},{-40,-40}})));
    LightArc lightArc annotation (Placement(transformation(
          extent={{-10,-10},{10,10}},
          rotation=90,
          origin={50,0})));
    Modelica.Electrical.Analog.Basic.Capacitor capacitor(v(fixed=true), C=5e-6)
      annotation (Placement(transformation(
          extent={{-10,-10},{10,10}},
          rotation=90,
          origin={10,0})));
  equation
    connect(constantCurrent.p, ground.p) annotation (Line(
        points={{-50,-10},{-50,-40}},
        color={0,0,255},
        smooth=Smooth.None));
    connect(capacitor.n, constantCurrent.n) annotation (Line(
        points={{10,10},{10,30},{-50,30},{-50,10}},
        color={0,0,255},
        smooth=Smooth.None));
    connect(capacitor.p, constantCurrent.p) annotation (Line(
        points={{10,-10},{10,-30},{-50,-30},{-50,-10}},
        color={0,0,255},
        smooth=Smooth.None));
    connect(lightArc.n, constantCurrent.n) annotation (Line(
        points={{50,10},{50,30},{-50,30},{-50,10}},
        color={0,0,255},
        smooth=Smooth.None));
    connect(lightArc.p, constantCurrent.p) annotation (Line(
        points={{50,-10},{50,-30},{-50,-30},{-50,-10}},
        color={0,0,255},
        smooth=Smooth.None));
    annotation (                                 Diagram(coordinateSystem(
            preserveAspectRatio=false, extent={{-100,-100},{100,100}}), graphics));
  end testLightArc;

  annotation (uses(Modelica(version="3.2.1")));
end Excercise12;
